@@ -1,28 +1,7 @@
 # -*- coding: utf-8 -
 #
-# 2009 (c) Benoit Chesneau <benoitc@e-engura.com> 
-# 2009 (c) Paul J. Davis <paul.joseph.davis@gmail.com>
-#
-# Permission is hereby granted, free of charge, to any person
-# obtaining a copy of this software and associated documentation
-# files (the "Software"), to deal in the Software without
-# restriction, including without limitation the rights to use,
-# copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following
-# conditions:
-#
-# The above copyright notice and this permission notice shall be
-# included in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-# OTHER DEALINGS IN THE SOFTWARE.
+# This file is part of gunicorn released under the MIT license. 
+# See the NOTICE for more information.
 
 import errno
 import fcntl
@@ -34,10 +13,7 @@ import socket
 import sys
 import time
 
-from worker import Worker
-
-logging.basicConfig(level=logging.DEBUG, format="%(levelname)s: %(message)s")
-log = logging.getLogger(__name__)
+from gunicorn.worker import Worker
 
 class Arbiter(object):
     
@@ -49,11 +25,11 @@ class Arbiter(object):
     SIG_QUEUE = []
     SIGNALS = map(
         lambda x: getattr(signal, "SIG%s" % x),
-        "QUIT INT TERM TTIN TTOU".split()
+        "HUP QUIT INT TERM TTIN TTOU USR1 USR2 WINCH".split()
     )
     SIG_NAMES = dict(
         (getattr(signal, name), name[3:].lower()) for name in dir(signal)
-        if name[:3] == "SIG"
+        if name[:3] == "SIG" and name[3] != "_"
     )
     
     def __init__(self, address, num_workers, modname):
@@ -61,11 +37,14 @@ class Arbiter(object):
         self.num_workers = num_workers
         self.modname = modname
         self.timeout = 30
+        self.reexec_pid = 0
         self.pid = os.getpid()
+        self.log = logging.getLogger(__name__)
         self.init_signals()
         self.listen(self.address)
-        log.info("Booted Arbiter: %s" % os.getpid())
-
+        self.log.info("Booted Arbiter: %s" % os.getpid())
+        
+                    
     def init_signals(self):
         if self.PIPE:
             map(lambda p: p.close(), self.PIPE)
@@ -73,24 +52,34 @@ class Arbiter(object):
         map(self.set_non_blocking, pair)
         map(lambda p: fcntl.fcntl(p, fcntl.F_SETFD, fcntl.FD_CLOEXEC), pair)
         map(lambda s: signal.signal(s, self.signal), self.SIGNALS)
+        signal.signal(signal.SIGCHLD, self.handle_chld)
     
     def set_non_blocking(self, fd):
         flags = fcntl.fcntl(fd, fcntl.F_GETFL) | os.O_NONBLOCK
         fcntl.fcntl(fd, fcntl.F_SETFL, flags)
-    
+
     def signal(self, sig, frame):
         if len(self.SIG_QUEUE) < 5:
             self.SIG_QUEUE.append(sig)
+            self.wakeup()
         else:
-            log.warn("Ignoring rapid signaling: %s" % sig)
-        # Wake up the arbiter
-        try:
-            os.write(self.PIPE[1], '.')
-        except IOError, e:
-            if e.errno not in [errno.EAGAIN, errno.EINTR]:
-                raise
-    
+            self.log.warn("Ignoring rapid signaling: %s" % sig)
+        
+
     def listen(self, addr):
+        if 'GUNICORN_FD' in os.environ:
+            fd = int(os.environ['GUNICORN_FD'])
+            del os.environ['GUNICORN_FD']
+            try:
+                sock = self.init_socket_fromfd(fd, addr)
+                self.LISTENER = sock
+                return
+            except socket.error, e:
+                if e[0] == errno.ENOTCONN:
+                    self.log.error("should be a non GUNICORN environnement")
+                else:
+                    raise
+        
         for i in range(5):
             try:
                 sock = self.init_socket(addr)
@@ -98,65 +87,77 @@ class Arbiter(object):
                 break
             except socket.error, e:
                 if e[0] == errno.EADDRINUSE:
-                    log.error("Connection in use: %s" % str(addr))
+                    self.log.error("Connection in use: %s" % str(addr))
                 if i < 5:
-                    log.error("Retrying in 1 second.")
+                    self.log.error("Retrying in 1 second.")
                 time.sleep(1)
+                
+    def init_socket_fromfd(self, fd, address):
+        sock = socket.fromfd(fd, socket.AF_INET, socket.SOCK_STREAM)
+        self.set_sockopts(sock)
+        return sock
 
     def init_socket(self, address):
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self.set_sockopts(sock)
+        sock.bind(address)
+        sock.listen(2048)
+        return sock
+        
+    def set_sockopts(self, sock):
         sock.setblocking(0)
         sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
-        
         if hasattr(socket, "TCP_CORK"):
             sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_CORK, 1)
         elif hasattr(socket, "TCP_NOPUSH"):
             sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NOPUSH, 1)
-        sock.bind(address)
-        sock.listen(2048)
-        return sock
 
     def run(self):
         self.manage_workers()
         while True:
             try:
+                self.reap_workers()
                 sig = self.SIG_QUEUE.pop(0) if len(self.SIG_QUEUE) else None
-
                 if sig is None:
                     self.sleep()
+                    self.murder_workers()
+                    self.manage_workers()
                     continue
                 
                 if sig not in self.SIG_NAMES:
-                    log.info("Ignoring unknown signal: %s" % sig)
+                    self.log.info("Ignoring unknown signal: %s" % sig)
                     continue
                 
                 signame = self.SIG_NAMES.get(sig)
                 handler = getattr(self, "handle_%s" % signame, None)
                 if not handler:
-                    log.error("Unhandled signal: %s" % signame)
+                    self.log.error("Unhandled signal: %s" % signame)
                     continue
-
-                log.info("Handling signal: %s" % signame)
-                handler()
-                
-                self.murder_workers()
-                self.reap_workers()
-                self.manage_workers()
-
+                self.log.info("Handling signal: %s" % signame)
+                handler()  
+                self.wakeup()   
             except StopIteration:
                 break
             except KeyboardInterrupt:
                 self.stop(False)
                 sys.exit(-1)
-            except Exception, e:
-                log.exception("Unhandled exception in main loop.")
+            except Exception:
+                self.log.exception("Unhandled exception in main loop.")
                 self.stop(False)
                 sys.exit(-1)
-
-        log.info("Master is shutting down.")
+                
+        self.log.info("Master is shutting down.")
         self.stop()
-    
+        
+    def handle_chld(self, sig, frame):
+        self.wakeup()
+        
+    def handle_hup(self):
+        self.log.info("Master hang up.")
+        self.reexec()
+        raise StopIteration
+        
     def handle_quit(self):
         raise StopIteration
     
@@ -174,10 +175,31 @@ class Arbiter(object):
     def handle_ttou(self):
         if self.num_workers > 0:
             self.num_workers -= 1
+            
+    def handle_usr1(self):
+        self.kill_workers(signal.SIGUSR1)
     
+    def handle_usr2(self):
+        self.reexec()
+        
+    def handle_winch(self):
+        if os.getppid() == 1 or os.getpgrp() != os.getpid():
+            self.logger.info("graceful stop of workers")
+            self.kill_workers(True)
+        else:
+            self.log.info("SIGWINCH ignored. not daemonized")
+    
+    def wakeup(self):
+        # Wake up the arbiter
+        try:
+            os.write(self.PIPE[1], '.')
+        except IOError, e:
+            if e.errno not in [errno.EAGAIN, errno.EINTR]:
+                raise
+                    
     def sleep(self):
         try:
-            ready = select.select([self.PIPE[0]], [], [], 1)
+            ready = select.select([self.PIPE[0]], [], [], 1.0)
             if not ready[0]:
                 return
             while os.read(self.PIPE[0], 1):
@@ -192,7 +214,7 @@ class Arbiter(object):
             sys.exit()
     
     def stop(self, graceful=True):
-        self.LISTENER.close()
+        self.LISTENER = None
         sig = signal.SIGQUIT
         if not graceful:
             sig = signal.SIGTERM
@@ -202,24 +224,33 @@ class Arbiter(object):
             time.sleep(0.1)
             self.reap_workers()
         self.kill_workers(signal.SIGKILL)
-    
+        
+    def reexec(self):
+        self.reexec_pid = os.fork()
+        if self.reexec_pid == 0:
+            os.environ['GUNICORN_FD'] = str(self.LISTENER.fileno())
+            os.execlp(sys.argv[0], *sys.argv)
+
     def murder_workers(self):
-        for (pid, worker) in self.WORKERS.iteritems():
-            diff = time.time() - os.fstat(worker.tmp.fileno()).st_mtime
-            if diff < self.timeout:
+        for (pid, worker) in list(self.WORKERS.items()):
+            diff = time.time() - os.fstat(worker.tmp.fileno()).st_ctime
+            if diff <= self.timeout:
                 continue
+            self.log.error("worker %s PID %s timeout killing." % (str(worker.id), pid))
             self.kill_worker(pid, signal.SIGKILL)
     
     def reap_workers(self):
         try:
             while True:
                 wpid, status = os.waitpid(-1, os.WNOHANG)
-                if not wpid:
-                    break
-                worker = self.WORKERS.pop(wpid, None)
-                if not worker:
-                    continue
-                worker.tmp.close()
+                if not wpid: break
+                if self.reexec_pid == wpid:
+                    self.reexec_pid = 0
+                else:
+                    worker = self.WORKERS.pop(wpid, None)
+                    if not worker:
+                        continue
+                    worker.tmp.close()
         except OSError, e:
             if e.errno == errno.ECHILD:
                 pass
@@ -238,24 +269,28 @@ class Arbiter(object):
             if i in workers:
                 continue
 
-            worker = Worker(i, self.pid, self.LISTENER, self.modname)
+            worker = Worker(i, self.pid, self.LISTENER, self.modname,
+                        self.timeout)
             pid = os.fork()
             if pid != 0:
                 self.WORKERS[pid] = worker
                 continue
             
             # Process Child
+            worker_pid = os.getpid()
             try:
-                log.info("Worker %s booting" % os.getpid())
+                self.log.info("Worker %s booting" % worker_pid)
                 worker.run()
                 sys.exit(0)
             except SystemExit:
+                
                 raise
             except:
-                log.exception("Exception in worker process.")
+                self.log.exception("Exception in worker process.")
                 sys.exit(-1)
             finally:
-                log.info("Worker %s exiting." % os.getpid())
+                worker.tmp.close()
+                self.log.info("Worker %s exiting." % worker_pid)
 
     def kill_workers(self, sig):
         for pid in self.WORKERS.keys():
@@ -264,10 +299,14 @@ class Arbiter(object):
     def kill_worker(self, pid, sig):
         worker = self.WORKERS.pop(pid)
         try:
-            os.kill(pid, sig) 
+            os.kill(pid, sig)
+            kpid, stat = os.waitpid(pid, os.WNOHANG)
+            if kpid:
+                self.log.warning("Problem killing process: %s" % pid)
         except OSError, e:
             if e.errno == errno.ESRCH:
-                pass
-        finally:
-            worker.tmp.close()
+                try:
+                    worker.tmp.close()
+                except:
+                    pass
 
