@@ -1,37 +1,70 @@
 # -*- coding: utf-8 -
 #
-# 2009 (c) Benoit Chesneau <benoitc@e-engura.com> 
-# 2009 (c) Paul J. Davis <paul.joseph.davis@gmail.com>
-#
-# Permission is hereby granted, free of charge, to any person
-# obtaining a copy of this software and associated documentation
-# files (the "Software"), to deal in the Software without
-# restriction, including without limitation the rights to use,
-# copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following
-# conditions:
-#
-# The above copyright notice and this permission notice shall be
-# included in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-# OTHER DEALINGS IN THE SOFTWARE.
+# This file is part of gunicorn released under the MIT license. 
+# See the NOTICE for more information.
 
+import errno
+import fcntl
+import select
+import socket
 import time
 
-weekdayname = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']
 
+timeout_default = object()
+
+CHUNK_SIZE = (16 * 1024)
+
+MAX_BODY = 1024 * (80 + 32)
+
+weekdayname = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']
 monthname = [None,
              'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
              'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
 
+def close_on_exec(fd):
+    flags = fcntl.fcntl(fd, fcntl.F_GETFD) | fcntl.FD_CLOEXEC
+    fcntl.fcntl(fd, fcntl.F_SETFL, flags)
+
+def close(sock):
+    try:
+        sock.close()
+    except socket.error:
+        pass
+  
+def read_partial(sock, length):
+    while True:
+        try:
+            ret = select.select([sock.fileno()], [], [], 0)
+            if ret[0]: break
+        except select.error, e:
+            if e[0] == errno.EINTR:
+                continue
+            raise
+    data = sock.recv(length)
+    return data
+
+    
+def write(sock, data):
+    buf = ""
+    buf += data
+    i = 0
+    while buf:
+        try:
+            bytes = sock.send(buf)
+            if bytes < len(buf):
+                buf = buf[bytes:]
+                continue
+            return len(data)
+        except socket.error, e:
+            if e[0] in (errno.EWOULDBLOCK, errno.EAGAIN):
+                break
+            raise
+        i += 1
+
+
+def normalize_name(name):
+    return  "-".join([w.lower().capitalize() for w in name.split("-")])
+    
 def import_app(module):
     parts = module.rsplit(":", 1)
     if len(parts) == 1:
@@ -61,4 +94,4 @@ def http_date(timestamp=None):
             weekdayname[wd],
             day, monthname[month], year,
             hh, mm, ss)
-    return s
\ No newline at end of file
+    return s
